package org.rcisoft.business.cmspriceclick.controller;


/*固定导入*/

import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import com.alipay.api.internal.util.file.IOUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.rcisoft.business.cmspriceclick.entity.CmsPriceClick;
import org.rcisoft.business.cmspriceclick.entity.CmsPriceClickDTO;
import org.rcisoft.business.cmspriceclick.entity.WxUserImageDTO;
import org.rcisoft.business.cmspriceclick.service.CmsPriceClickService;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.List;


/**
 * Created by cy on 2024年6月3日 下午3:35:01.
 */
@RestController
@RequestMapping("/cmspriceclick")
public class CmsPriceClickController extends CyPaginationController<CmsPriceClick> {
    private static final String PRICE_CLICK_IMPORT_TEMPLATE_NAME = "PromotionalTemplates.xls";
    @Autowired
    private Global global;
    @Autowired
    private CmsPriceClickService cmsPriceClickServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:priceClick:add')")
    @CyOpeLogAnno(title = "system-文章/视频单价点击量表管理-新增文章/视频单价点击量表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value = "添加文章/视频单价点击量表", notes = "添加文章/视频单价点击量表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid CmsPriceClick cmsPriceClick, BindingResult bindingResult) {
        CyPersistModel data = cmsPriceClickServiceImpl.persist(cmsPriceClick);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsPriceClick);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:priceClick:delete')")
    @CyOpeLogAnno(title = "system-文章/视频单价点击量表管理-删除文章/视频单价点击量表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "逻辑删除文章/视频单价点击量表", notes = "逻辑删除文章/视频单价点击量表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId, CmsPriceClick cmsPriceClick) {
        cmsPriceClick.setBusinessId(businessId);
        CyPersistModel data = cmsPriceClickServiceImpl.removeLogical(cmsPriceClick);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:priceClick:delete')")
    @CyOpeLogAnno(title = "system-文章/视频单价点击量表管理-删除文章/视频单价点击量表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "删除文章/视频单价点击量表", notes = "删除文章/视频单价点击量表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId, CmsPriceClick cmsPriceClick) {
        cmsPriceClick.setBusinessId(businessId);
        CyPersistModel data = cmsPriceClickServiceImpl.remove(cmsPriceClick);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:priceClick:update')")
    @CyOpeLogAnno(title = "system-文章/视频单价点击量表管理-修改文章/视频单价点击量表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改文章/视频单价点击量表", notes = "修改文章/视频单价点击量表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid CmsPriceClick cmsPriceClick, BindingResult bindingResult) {
        cmsPriceClick.setBusinessId(businessId);
        CyPersistModel data = cmsPriceClickServiceImpl.merge(cmsPriceClick);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsPriceClick);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:priceClick:query')")
    @CyOpeLogAnno(title = "system-文章/视频单价点击量表管理-查询文章/视频单价点击量表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询单一文章/视频单价点击量表", notes = "查询单一文章/视频单价点击量表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsPriceClickServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:priceClick:list')")
    @CyOpeLogAnno(title = "system-文章/视频单价点击量表管理-查询文章/视频单价点击量表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询文章/视频单价点击量表集合", notes = "查询文章/视频单价点击量表集合")
    @GetMapping(value = "/listAll")
    public CyResult listAll(CmsPriceClick cmsPriceClick) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsPriceClickServiceImpl.findAll(cmsPriceClick));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:priceClick:list')")
    /*@CyOpeLogAnno(title = "system-文章/视频单价点击量表管理-查询文章/视频单价点击量表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询文章/视频单价点击量表集合", notes="分页查询文章/视频单价点击量表集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(CmsPriceClick cmsPriceClick) {
        cmsPriceClickServiceImpl.findAllByPagination(getPaginationUtility(), cmsPriceClick);
        return getGridModelResponse();
    }*/

    @CyOpeLogAnno(title = "system-文章/视频单价点击量表管理-查询文章/视频单价点击量表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出文章/视频单价点击量表信息", notes = "导出文章/视频单价点击量表信息")
    @GetMapping(value = "/export")
    public void outCmsPriceClick(HttpServletResponse response, CmsPriceClick cmsPriceClick, Integer unitId) {
        String excelName = "推广次数信息.xlsx";
        cmsPriceClick.setUnitId(unitId);
        List<CmsPriceClickDTO> cmsPriceClickList = cmsPriceClickServiceImpl.export(cmsPriceClick);
        //循环设置序号
        for (CmsPriceClickDTO cmsPriceClickDTO : cmsPriceClickList) {
            cmsPriceClickDTO.setNumber(cmsPriceClickList.indexOf(cmsPriceClickDTO) + 1);
        }

        CyEpExcelUtil.exportExcel(cmsPriceClickList, "推广次数信息", "推广次数信息", CmsPriceClickDTO.class, excelName, response);
    }

    /*---------------------------------------------------------------------------------------------------*/

    /**
     * 分页查询阅读量管理
     */
    @CyOpeLogAnno(title = "system-阅读量管理-查询阅读量管理", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "分页查询阅读量管理", notes = "分页查询阅读量管理")
    @GetMapping(value = "/list")
    public IPage<CmsPriceClickDTO> listByPagination(CmsPriceClickDTO cmsPriceClickDTO) {
        return cmsPriceClickServiceImpl.paginateTheNumberOfViews(getPaginationUtility(), cmsPriceClickDTO);
    }

    /**
     * 修改当前这条数据的阅读量以或者价格
     */
    @CyOpeLogAnno(title = "system-阅读量管理-修改阅读量管理", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改当前这条数据的阅读量以或者价格", notes = "修改当前这条数据的阅读量以或者价格")
    @PutMapping(value = "/update")
    public CyResult update(CmsPriceClickDTO cmsPriceClickDTO) {
        CyPersistModel data = cmsPriceClickServiceImpl.updatebyId(cmsPriceClickDTO);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsPriceClickDTO);
    }

    /**
     * 导入修改
     */
    @CyOpeLogAnno(title = "system-阅读量管理-导入修改", businessType = CyLogTypeEnum.IMPORT)
    @ApiOperation(value = "导入修改", notes = "导入修改")
    @PostMapping(value = "/import")
    public CyResult importExcel(MultipartFile file) {
        //校验文件是否为空
        if (file.isEmpty()) {
            return CyResultGenUtil.builder(new CyPersistModel(0), "导入失败，文件为空",
                    "导入失败，文件为空", null);
        }
        //校验文件是否为excel
        String fileName = file.getOriginalFilename();
        if (fileName != null && !fileName.endsWith(".xls") && !fileName.endsWith(".xlsx")) {
            return CyResultGenUtil.builder(new CyPersistModel(0), "导入失败，文件格式不正确",
                    "导入失败，文件格式不正确", null);
        }
        CyPersistModel data = cmsPriceClickServiceImpl.importExcel(file);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                null);
    }

    /**
     * 导入增加
     */
    @CyOpeLogAnno(title = "system-阅读量管理-导入增加", businessType = CyLogTypeEnum.IMPORT)
    @ApiOperation(value = "导入增加", notes = "导入增加")
    @PostMapping(value = "/importAdd")
    public CyResult importExcelAdd(MultipartFile file, CmsPriceClick cmsPriceClick) {
        //校验文件是否为空
        if (file.isEmpty()) {
            return CyResultGenUtil.builder(new CyPersistModel(0), "导入失败，文件为空",
                    "导入失败，文件为空", null);
        }
        //校验文件是否为excel
        String fileName = file.getOriginalFilename();
        if (fileName != null && !fileName.endsWith(".xls") && !fileName.endsWith(".xlsx")) {
            return CyResultGenUtil.builder(new CyPersistModel(0), "导入失败，文件格式不正确",
                    "导入失败，文件格式不正确", null);
        }
        cmsPriceClickServiceImpl.importExcelAdd(file, cmsPriceClick);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                null);

    }

    /**
     * 推广表导出模板
     */
    @CyOpeLogAnno(title = "system-阅读量管理-导出阅读量管理模版", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出阅读量管理模版", notes = "导出阅读量管理模版")
    @GetMapping(value = "/exportTemplate")
    public void exportTemplate(HttpServletResponse response) {
        OutputStream os = null;
        Workbook workbook = null;
        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("推广导入模板.xlsx", "UTF-8"));
            ExportParams exportParams = new ExportParams("推广导入模板", "推广导入模板");
            exportParams.setType(ExcelType.XSSF);
            InputStream resourceAsStream = getClass().getClassLoader().getResourceAsStream(global.getEXCEL_TEMPLATE_LOCATION() + PRICE_CLICK_IMPORT_TEMPLATE_NAME);
            //workbook = ExcelExportUtil.exportExcel(exportParams,SysUser.class, new ArrayList<>());
//            String s = global.getEXCEL_TEMPLATE_LOCATION() + STUDENT_IMPORT_TEMPLATE_NAME;
            workbook = WorkbookFactory.create(resourceAsStream);
            os = response.getOutputStream();
            workbook.write(os);
        } catch (Exception e) {
            System.out.println("------------------------打印异常日志--------------------------------");
            System.out.println(e);
            throw new CyServiceException("下载模版失败");
        } finally {
            IOUtils.closeQuietly(workbook);
            IOUtils.closeQuietly(os);
        }
    }

    //微信小程序根据商圈查代言人信息
    @ApiOperation(value = "微信小程序根据商圈查代言人信息", notes = "微信小程序根据商圈查代言人信息")
    @GetMapping(value = "/getCmsPriceClickByCircleId")
    public CyResult getCmsPriceClickByCircleId(WxUserImageDTO wxUserImageDTO) {
        IPage<WxUserImageDTO> cmsPriceClickByCircleId = cmsPriceClickServiceImpl.getCmsPriceClickByCircleId(getPaginationUtility(), wxUserImageDTO);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR, cmsPriceClickByCircleId);
    }

}
