package org.rcisoft.business.cmspriceclick.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.cmsbankcard.service.CmsBankCardService;
import org.rcisoft.business.cmspriceclick.dao.CmsPriceClickRepository;
import org.rcisoft.business.cmspriceclick.entity.CmsPriceClick;
import org.rcisoft.business.cmspriceclick.entity.CmsPriceClickDTO;
import org.rcisoft.business.cmspriceclick.entity.WxUserImageDTO;
import org.rcisoft.business.cmspriceclick.service.CmsPriceClickService;
import org.rcisoft.business.sysunit.dao.SysUnitRepository;
import org.rcisoft.business.sysunit.entity.SysUnit;
import org.rcisoft.business.sysunit.service.impl.SysUnitServiceImpl;
import org.rcisoft.business.sysuser.emuns.AdminUserInfoExceptionEnums;
import org.rcisoft.business.sysuser.entity.AddSysUser;
import org.rcisoft.business.sysuser.entity.SysUser;
import org.rcisoft.business.sysuser.service.impl.SysUserServiceImpl;
import org.rcisoft.core.component.CyInitComp;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.wbac.user.dao.SysUserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigInteger;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * Created by cy on 2024年6月3日 下午3:35:01.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CmsPriceClickServiceImpl extends ServiceImpl<CmsPriceClickRepository, CmsPriceClick> implements CmsPriceClickService {

    @Autowired
    private CyInitComp cyInitComp;
    @Autowired
    private SysUserServiceImpl sysUserServiceImpl;
    @Autowired
    private SysUnitServiceImpl sysUnitServiceImpl;
    @Autowired
    private SysUnitRepository sysUnitRepository;

    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private SysUserRepository sysUserRepository;
    @Autowired
    private CmsBankCardService cmsBankCardService;

    /**
     * 保存 文章/视频单价点击量表
     *
     * @param cmsPriceClick
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(CmsPriceClick cmsPriceClick) {
        //增加操作
        int line = baseMapper.insert(cmsPriceClick);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                cmsPriceClick.getBusinessId() + "的文章/视频单价点击量表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 文章/视频单价点击量表
     *
     * @param cmsPriceClick
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(CmsPriceClick cmsPriceClick) {
        int line = baseMapper.realDelete(cmsPriceClick);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                cmsPriceClick.getBusinessId() + "的文章/视频单价点击量表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 文章/视频单价点击量表
     *
     * @param cmsPriceClick
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(CmsPriceClick cmsPriceClick) {
        cmsPriceClick.setDeleted();
        int line = baseMapper.deleteById(cmsPriceClick);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                cmsPriceClick.getBusinessId() + "的文章/视频单价点击量表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 文章/视频单价点击量表
     *
     * @param cmsPriceClick
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(CmsPriceClick cmsPriceClick) {
        int line = baseMapper.updateById(cmsPriceClick);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + cmsPriceClick.getBusinessId() + "的文章/视频单价点击量表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 文章/视频单价点击量表
     *
     * @param id
     * @return
     */
    @Override
    public CmsPriceClick findById(int id) {
        return baseMapper.selectById(id);
    }

    /**
     * 分页查询 文章/视频单价点击量表
     *
     * @param cmsPriceClick
     * @return
     */
    @Override
    public IPage<CmsPriceClick> findAllByPagination(CyPageInfo<CmsPriceClick> paginationUtility,
                                                    CmsPriceClick cmsPriceClick) {
        return baseMapper.queryCmsPriceClicksPaged(paginationUtility, cmsPriceClick);

    }

    /**
     * 查询list 文章/视频单价点击量表
     *
     * @param cmsPriceClick
     * @return
     */
    @Override
    public List<CmsPriceClickDTO> findAll(CmsPriceClick cmsPriceClick) {
        return baseMapper.queryCmsPriceClicks(cmsPriceClick);

    }

    /**
     * 导出文章/视频单价点击量表
     *
     * @return
     */
    @Override
    public List<CmsPriceClickDTO> export(CmsPriceClick cmsPriceClick) {
        List<CmsPriceClickDTO> cmsPriceClickList = baseMapper.queryCmsPriceClicks(cmsPriceClick);
        return cmsPriceClickList;
    }

    @Override
    public IPage<CmsPriceClickDTO> paginateTheNumberOfViews(CyPageInfo<CmsPriceClick> paginationUtility, CmsPriceClickDTO cmsPriceClickDTO) {
        //判断当前用户角色 来决定数据范围 1 超级管理员 2 商家管理员
        String authenRoleIds = CyUserUtil.getAuthenRoleIds();
        //authenRoleIds的场景只会存在一个数据 判断
        // 1 = 超级管理员
        if (cyInitComp.admin[0].equals(authenRoleIds)) {
            //分页配置
            paginationUtility.setSize(cmsPriceClickDTO.getRows());
            paginationUtility.setCurrent(cmsPriceClickDTO.getPage());
            return baseMapper.superAdminPaginatedQuery(paginationUtility, cmsPriceClickDTO);
        }
        // 2 = 商家管理员
        if (cyInitComp.unit[0].equals(authenRoleIds)) {
            //分页配置
            paginationUtility.setSize(cmsPriceClickDTO.getRows());
            paginationUtility.setCurrent(cmsPriceClickDTO.getPage());
            return baseMapper.merchantAdminsPaginateQueries(paginationUtility, cmsPriceClickDTO);
        }
        return null;
    }

    @Override
    public CyPersistModel updatebyId(CmsPriceClickDTO cmsPriceClickDTO) {
        int i = baseMapper.updateById(cmsPriceClickDTO);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了了ID为" +
                cmsPriceClickDTO.getBusinessId() + "的文章/视频单价点击量表信息");
        return new CyPersistModel(i);
    }

    @Override
    public void updateTheUserByMerchantId(SysUnit sysUnit) {
        baseMapper.updateTheUserByMerchantId(sysUnit);
    }

    @Override
    public CyPersistModel importExcel(MultipartFile file) {
        //导入修改 导入的文件解析成为对象集合
        List<CmsPriceClickDTO> cmsPriceClicksDTOList = CyEpExcelUtil.importExcel(file, 0, 2, CmsPriceClickDTO.class);
        int insert = 0;
        //一条一条改
        for (CmsPriceClickDTO cmsPriceClickDTO : cmsPriceClicksDTOList) {//编号找到当前信息
            //判断当前类型 是文章还是视频
            if (("文章").equals(cmsPriceClickDTO.getType())) {
                CmsPriceClick cmsPriceClick = baseMapper.findByTitleAndUnitNameAndLoginNameNew(cmsPriceClickDTO);
                //判断推广次数或及价格是否有修改
                if (cmsPriceClick != null) {
                    if (cmsPriceClick.getClickThroughRate() != cmsPriceClickDTO.getClickThroughRate() ||
                            cmsPriceClick.getPrice() != cmsPriceClickDTO.getPrice()) {
                        cmsPriceClick.setClickThroughRate(cmsPriceClickDTO.getClickThroughRate());
                        cmsPriceClick.setPrice(cmsPriceClickDTO.getPrice());
                        insert = baseMapper.updateById(cmsPriceClick);
                    }
                }
            }
            if (("视频").equals(cmsPriceClickDTO.getType())) {
                CmsPriceClick cmsPriceClick = baseMapper.findByTitleAndUnitNameAndLoginNameVideo(cmsPriceClickDTO);
                if (cmsPriceClick != null) {
                    if (cmsPriceClick.getClickThroughRate() != cmsPriceClickDTO.getClickThroughRate() ||
                            cmsPriceClick.getPrice() != cmsPriceClickDTO.getPrice()) {
                        cmsPriceClick.setClickThroughRate(cmsPriceClickDTO.getClickThroughRate());
                        cmsPriceClick.setPrice(cmsPriceClickDTO.getPrice());
                        insert = baseMapper.updateById(cmsPriceClick);
                    }
                }
            }
        }
        return new CyPersistModel(insert);
    }

    @Override
    public void importExcelAdd(MultipartFile file, CmsPriceClick cmsPriceClicks) {
        //将文件解析成为对象集合
        List<CmsPriceClickDTO> cmsPriceClicksDTOList = CyEpExcelUtil.importExcel(file, 0, 2, CmsPriceClickDTO.class);
        cmsPriceClicksDTOList.stream().filter(Objects::nonNull).collect(Collectors.toList());
        //
        Pattern pattern = Pattern.compile("[0-9]*");
        //行数
        Integer sum = 0;
        //去除cmsPriceClicksDTOList里属性为null的数据
        cmsPriceClicksDTOList = cmsPriceClicksDTOList.stream().filter(cmsPriceClickDTO -> {
            if (cmsPriceClickDTO.getUserName() != null && cmsPriceClickDTO.getPhone() != null && cmsPriceClickDTO.getIdCard() != null && cmsPriceClickDTO.getBankCard() != null) {
                return true;
            }
            return false;
        }).collect(Collectors.toList());

        //添加前先遍历一下那些数据有问题
        for (CmsPriceClickDTO cmsPriceClickDTO : cmsPriceClicksDTOList) {
            sum++;
            //用户名去空格
            cmsPriceClickDTO.setUserName(cmsPriceClickDTO.getUserName().replaceAll("\\s+", "").trim());
            cmsPriceClickDTO.setUserName(cmsPriceClickDTO.getUserName().replaceAll("\\p{Zs}", "").trim());
            //手机号去空格
            cmsPriceClickDTO.setPhone(cmsPriceClickDTO.getPhone().replaceAll("\\s+", "").trim());
            cmsPriceClickDTO.setPhone(cmsPriceClickDTO.getPhone().replaceAll("\\p{Zs}", "").trim());
            //身份证号去空格
            cmsPriceClickDTO.setIdCard(cmsPriceClickDTO.getIdCard().replaceAll("\\s+", "").trim());
            cmsPriceClickDTO.setIdCard(cmsPriceClickDTO.getIdCard().replaceAll("\\p{Zs}", "").trim());
            //银行卡号去空格
            cmsPriceClickDTO.setBankCard(cmsPriceClickDTO.getBankCard().replaceAll("\\s+", "").trim());
            cmsPriceClickDTO.setBankCard(cmsPriceClickDTO.getBankCard().replaceAll("\\p{Zs}", "").trim());

            //1.用户身份证号不能为空   18位 且必须为数字类型
            if (cmsPriceClickDTO.getIdCard() == null || cmsPriceClickDTO.getIdCard().length() != 18) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.ID_LENGTH_ERROR.getCode(), sum + "行" + AdminUserInfoExceptionEnums.ID_LENGTH_ERROR.getMessage());
            }
            //2.用户手机号    11位
            if (cmsPriceClickDTO.getPhone() == null || cmsPriceClickDTO.getPhone().length() != 11) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.ID_LENGTH_ERROR.getCode(), sum + "行" + AdminUserInfoExceptionEnums.PHONE_LENGTH_ERROR.getMessage());
            }
            //3.银行卡号不能为空   30位
            if (cmsPriceClickDTO.getBankCard() == null || cmsPriceClickDTO.getBankCard().length() > 30) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.BANK_LENGTH_ERROR.getCode(), sum + "行" + AdminUserInfoExceptionEnums.BANK_LENGTH_ERROR.getMessage());
            }
            //4.推广次数不能为空
            if (cmsPriceClickDTO.getClickThroughRate() == null) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.CLICK_THROUGH_RATE_ERROR.getCode(), sum + "行" + AdminUserInfoExceptionEnums.CLICK_THROUGH_RATE_ERROR.getMessage());
            }
            //5.价格不能为空
            if (cmsPriceClickDTO.getPrice() == null) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.PRICE_ERROR.getCode(), sum + "行" + AdminUserInfoExceptionEnums.PRICE_ERROR.getMessage());
            }
            //手机号只能是数字
            if (!pattern.matcher(cmsPriceClickDTO.getPhone()).matches()) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.PHONE_LENGTH_ERROR.getCode(), sum + "行" + AdminUserInfoExceptionEnums.PHONE_LENGTH_ERROR.getMessage());
            }
            //银行卡号只能是数字
            if (!pattern.matcher(cmsPriceClickDTO.getBankCard()).matches()) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.BANK_LENGTH_ERROR.getCode(), sum + "行" + AdminUserInfoExceptionEnums.BANK_LENGTH_ERROR.getMessage());
            }
            //用户名称不能为空且 不能大于30
            if (cmsPriceClickDTO.getUserName() == null || cmsPriceClickDTO.getUserName().length() > 30) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.USER_NAME_LENGTH_ERROR.getCode(), sum + "行" + AdminUserInfoExceptionEnums.USER_NAME_LENGTH_ERROR.getMessage());
            }
        }


        //查询出身份账号绑定用户
        ArrayList<SysUser> sysUserArrayList = sysUserServiceImpl.selectIdcardBid();
        //身份证号 与 用户 id
        HashMap<String, Integer> idCardBidHM = new HashMap<>();
        for (SysUser sysUser : sysUserArrayList) {
            idCardBidHM.put(sysUser.getIdNumber(), sysUser.getBusinessId());
        }
        //查出当前商家下是否有当前代言人
        ArrayList<Integer> userIdList = sysUnitServiceImpl.findUserId(cmsPriceClicks.getUnitId());
        //当前商家下当前新闻或视频 存在的数据
        ArrayList<CmsPriceClick> userIdListBid = baseMapper.selectUserIdBid(cmsPriceClicks.getUnitId(), cmsPriceClicks.getNewsVideoId(), cmsPriceClicks.getType());
        // 用户 id 与 这条数据
        HashMap<Integer, CmsPriceClick> userIdCmsPriceClickHashMap = new HashMap<>();
        for (CmsPriceClick cmsPriceClick : userIdListBid) {
            if (cmsPriceClick.getUserId() != null) {
                userIdCmsPriceClickHashMap.put(cmsPriceClick.getUserId(), cmsPriceClick);
            }
        }

        for (CmsPriceClickDTO cmsPriceClickDTO : cmsPriceClicksDTOList) {
            //用户是否拥有身份证号信息
            if (cmsPriceClickDTO.getIdCard() != null) {
                //判断当前用户是否在当前商家下
                if (userIdList.contains(idCardBidHM.get(cmsPriceClickDTO.getIdCard()))) {
                    CmsPriceClick cmsPriceClick = new CmsPriceClick();
                    cmsPriceClick.setUnitId(cmsPriceClicks.getUnitId());
                    cmsPriceClick.setNewsVideoId(cmsPriceClicks.getNewsVideoId());
                    cmsPriceClick.setType(cmsPriceClicks.getType());
                    cmsPriceClick.setUserId(idCardBidHM.get(cmsPriceClickDTO.getIdCard()));
                    cmsPriceClick.setClickThroughRate(cmsPriceClickDTO.getClickThroughRate());
                    cmsPriceClick.setPrice(cmsPriceClickDTO.getPrice());
                    cmsPriceClick.setCreateDate(new Date());
                    cmsPriceClick.setUpdateDate(new Date());
                    cmsPriceClick.setFlag(String.valueOf(1));
                    baseMapper.insert(cmsPriceClick);
                } else {
                    if (idCardBidHM.get(cmsPriceClickDTO.getIdCard()) == null) {
                        //1.没有当前代言人 ---》创建代言人 并将代言人添加到当前商家下，最后在新建 推广数据
                        //1.1   创建当前用户
                        AddSysUser addSysUser = new AddSysUser();
                        addSysUser.setUsername(cmsPriceClickDTO.getIdCard());
                        //类型
                        addSysUser.setUserType(String.valueOf(1));
                        //名称
                        addSysUser.setName(cmsPriceClickDTO.getUserName());
                        //手机号
                        addSysUser.setPhone(cmsPriceClickDTO.getPhone());
                        //密码为 身份证号后6为
                        addSysUser.setPassword(cmsPriceClickDTO.getIdCard().substring(cmsPriceClickDTO.getIdCard().length() - 6));
                        //身份证号
                        addSysUser.setIdNumber(cmsPriceClickDTO.getIdCard());
                        addSysUser.setFlag("1");
                        addSysUser.setIdentity("3");
                        //银行卡
                        addSysUser.setBankCardNum(cmsPriceClickDTO.getBankCard());
                        //配置角色
                        addSysUser.setRoleList(new ArrayList<>(List.of(3)));
                        //配置商家
                        addSysUser.setDeptId(BigInteger.valueOf(cmsPriceClicks.getUnitId()));
                        sysUserServiceImpl.persist(addSysUser);
                        //没有 新增
                        CmsPriceClick cmsPriceClick = new CmsPriceClick();
                        cmsPriceClick.setUnitId(cmsPriceClicks.getUnitId());
                        cmsPriceClick.setNewsVideoId(cmsPriceClicks.getNewsVideoId());
                        cmsPriceClick.setType(cmsPriceClicks.getType());
                        cmsPriceClick.setUserId(addSysUser.getBusinessId());
                        cmsPriceClick.setClickThroughRate(cmsPriceClickDTO.getClickThroughRate());
                        cmsPriceClick.setPrice(cmsPriceClickDTO.getPrice());
                        cmsPriceClick.setCreateDate(new Date());
                        cmsPriceClick.setUpdateDate(new Date());
                        cmsPriceClick.setFlag(String.valueOf(1));
                        baseMapper.insert(cmsPriceClick);
                        sysUserArrayList = sysUserServiceImpl.selectIdcardBid();
                        //身份证号 与 用户 id
                        for (SysUser sysUser : sysUserArrayList) {
                            idCardBidHM.put(sysUser.getIdNumber(), sysUser.getBusinessId());
                        }
                    } else {
                        //2.代言人不在当前商家下 代言人与商家绑定
                        sysUnitRepository.insertByUnitUser(cmsPriceClicks.getUnitId(), idCardBidHM.get(cmsPriceClickDTO.getIdCard()));
                        //没有 新增
                        CmsPriceClick cmsPriceClick = new CmsPriceClick();
                        cmsPriceClick.setUnitId(cmsPriceClicks.getUnitId());
                        cmsPriceClick.setNewsVideoId(cmsPriceClicks.getNewsVideoId());
                        cmsPriceClick.setType(cmsPriceClicks.getType());
                        cmsPriceClick.setUserId(idCardBidHM.get(cmsPriceClickDTO.getIdCard()));
                        cmsPriceClick.setClickThroughRate(cmsPriceClickDTO.getClickThroughRate());
                        cmsPriceClick.setPrice(cmsPriceClickDTO.getPrice());
                        cmsPriceClick.setCreateDate(new Date());
                        cmsPriceClick.setUpdateDate(new Date());
                        cmsPriceClick.setFlag(String.valueOf(1));
                        baseMapper.insert(cmsPriceClick);
                    }
                }
            }
        }
    }

    @Override
    public IPage<WxUserImageDTO> getCmsPriceClickByCircleId(CyPageInfo<CmsPriceClick> paginationUtility, WxUserImageDTO wxUserImageDTO) {
        paginationUtility.setSize(wxUserImageDTO.getRows());
        paginationUtility.setCurrent(wxUserImageDTO.getPage());
        return baseMapper.getCmsPriceClickByCircleId(paginationUtility, wxUserImageDTO);


    }
}
